#   Copyright (C) 2004 CCLRC & NERC( Natural Environment Research Council ).
#   This software may be distributed under the terms of the
#   Q Public License, version 1.0 or later. http://ndg.nerc.ac.uk/public_docs/QPublic_license.txt

"""
naError.py
==========

Container module for nappy error definitions.

"""

# Define global variables
NAPlatformError="Platform error: You cannot use the CDAT-dependent features on a Windows Platform."